local CowTip = _G.CowTip
local CowTip_LuaText = CowTip:GetModule("LuaText")

local L = setmetatable({}, { __index = function(self, index) return index end })

local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local function escape(text)
	return text:replace("|","||")
end

local function unescape(text)
	return text:replace("||", "|")
end

local function setLineText(info, value)
	local side = info[#info]
	local num = info.arg
	CowTip_LuaText.db.profile[side][num] = unescape(value)
end

local function getLineText(info)
	local side = info[#info]
	local num = info.arg
	return escape(CowTip_LuaText.db.profile[side][num])
end

local function getPreview(side, num)
	local text = CowTip_LuaText.db.profile[side][num]
	local prev = CowTip_LuaText:GetText(text, "player")
	print("prev: ", prev)
	return prev
end

local function getLeftPrev(info)
	return getPreview("lefts", info.arg)
end

local function getRightPrev(info)
	return getPreview("rights", info.arg)
end

local function makeLineOption(options, num)
	print("making options for line ", num)
	options.args["line" .. num] = {
		type = 'group',
		name = L["Line %d"]:format(num),
		desc = L["Line"],
		inline = true,
		order = num,
		args = {
			lefts = {
				type = 'input',
				name = L["left"],
				desc = L["left text"],
				multiline = true,
				width = "double",
				get = getLineText,
				set = setLineText,
				arg = num,
				order = 1,
			},
			rights = {
				type = 'input',
				name = L["right"],
				desc = L["right text"],
				multiline = true,
				width = "double",
				get = getLineText,
				set = setLineText,
				arg = num,
				order = 2,
			},
			leftPreview = {
				type = 'description',
				name = getLeftPrev,
				arg = num,
				width="double",
				order = 10,
			},
			rightPreview = {
				type = 'description',
				name = getRightPrev,
				arg = num,
				width="double",
				order = 11,
			},
		},
	}
end

local function showConfig()
	AceConfigDialog:SetDefaultSize("Cowtip_LuaText", 775, 500)
	AceConfigDialog:Open("Cowtip_LuaText")
	-- InterfaceOptionsFrame:Hide() -- hides everything...
end

local options
function CowTip_LuaText:CreateOptions()
	if options then
		return
	end
	options = {
		name = L["CowTip LuaText"],
		desc = L[""],
		type = 'group',
		args = {},
	}
	local dbp = self.db.profile
	for i = 1,30 do
		makeLineOption(options, i)
	end

	AceConfig:RegisterOptionsTable("Cowtip_LuaText", options)

	local options2 = {
		name = L["CowTip LuaText"],
		desc = L[""],
		type = 'group',
		args = {
			showConfig = {
				name = L["Open Configuration"],
				desc = L["Open standalone Configuration"],
				type = 'execute',
				func = showConfig,
			},
		},
	}
	
	AceConfig:RegisterOptionsTable("Cowtip_LuaText_bliz", options2)
	AceConfigDialog:AddToBlizOptions("Cowtip_LuaText_bliz", "Cowtip LuaText")
end
